#------------------------------------------------------------------------------
# Common shell functions for cl-* projects
# Includes: localization (_), formatted output (einfo, ebegin, eend),
# and help printing (print_opt, print_desk, etc.)
# Version: 20250715
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Version and Color Constants
#------------------------------------------------------------------------------
readonly VERSION="0.1.0"                                                                                  
# ANSI color codes for styled terminal output
readonly BBLUE='\e[1;34m'
readonly BGREEN='\e[1;32m'
readonly BRED='\e[1;31m'
readonly BYELLOW='\e[1;33m'
readonly RESET='\e[0m'

#------------------------------------------------------------------------------
# Localization helper function
#------------------------------------------------------------------------------

# Translation helper (gettext wrapper)
_() {
	# Get the translated string and print it with a newline
	local fmt=$(gettext "$1");
	shift
	printf "${fmt}\n" "$@"
}

#------------------------------------------------------------------------------
# Help Output Formatting Utilities
#------------------------------------------------------------------------------

# Print a top-level command description (used for headlines like "Syntax:")
print_cmd() {
    printf "%2s" ""     # Indent by 2 spaces
    _ "$@"              # Print translated string
}

# Print a short option (e.g. "-j, --jobs") with standard indent
print_opt() {
	local indent=24
    printf "%2s%-22s" "" "$1"   # Indent and print the option
    if (( ${#1} > 20 )); then   # If option string is too long, wrap to new line
        printf "\n"
		printf "%*s" "$indent" ""
    fi
}

# Print a long-only option (e.g. "--nocolor") with extra indent
print_opt_longonly() {
    printf "%6s%-18s" "" "$1"   # Indent and print the option
    if (( ${#1} > 20 )); then
        printf "\n"
		printf "%*s" "$indent" ""
    fi
}

# Print an indented description line (used after an option)
print_desk() {
	text=$(_ "$@")
	local indent=24 col=$(get_cols) line= first=
	local width=$((col - indent)) counter=0

	set -f
	for word in $text; do
		counter=$((counter+${#word}+1))

		if [[ $counter -le $width ]]; then
			[[ -z "$line" ]] || line+=' '
			line+="$word"
		else
			[[ -z $first ]] && first=1 || printf "%*s" "$indent" ""
			printf "$line\n"
			line=$word
			counter=${#word}
		fi
	done
	[[ -z $first ]] && first=1 || printf "%*s" "$indent" ""
	printf "$line\n"
	set +f
}

print_desk_next() {
	local indent=24
	printf "%*s" "$indent" ""
	print_desk "$@"
}

# Print an indented usage example (after a parameter description)
print_desk_example() {
    printf "%26s" ""    # Align to 26 characters
    echo "$@"           # Print raw example without translation
}

#------------------------------------------------------------------------------
# Informational and Status Output
#------------------------------------------------------------------------------

# Print an informational message with optional color
einfo() {
	local bgreen= reset=
    # Enable color if stdout is a terminal and color is allowed
	if [[ -t 1 && ${USE_COLOR:-1} -eq 1 ]]; then
		bgreen=$BGREEN
		reset=$RESET
	fi

	# Get translated message
	local fmt=$(gettext "$1");
	shift
	# Print the formatted message with an asterisk and optional color
	printf "${bgreen} * ${reset}${fmt}\n" "$@"
}

eerror() {
	local bred= reset=
	if [[ -t 1 && ${USE_COLOR:-1} -eq 1 ]]; then
		bgreen=$BRED
		reset=$RESET
	fi

	local fmt=$(gettext "$1");
	shift
	printf "${bred} * ${reset}${fmt}\n" "$@" >&2
}

# Print a "begin" message indicating the start of an operation
ebegin() {
	local bgreen= reset=
	# Enable color if stdout is a terminal and color is allowed
	if [[ -t 1 && ${USE_COLOR:-1} -eq 1 ]]; then
		bgreen=$BGREEN
		reset=$RESET
	fi
	# Add a newline only if output is a terminal
	local br=; [[ -t 1 ]] && br='\n'

	# Get translated message
	local fmt=$(gettext "$1");
	shift
	# Print "begin" message with ellipsis, optional color, and newline
	printf "${bgreen} * ${reset}${fmt} ...${br}" "$@"
}

# Print a status message with a colored marker (ok/!!),
# aligned to the right edge of the terminal.
eend() {
	local bblue= bgreen= bred= reset=
	if [[ -t 1 && ${USE_COLOR:-1} -eq 1 ]]; then
		bblue=$BBLUE
		bgreen=$BGREEN
		bred=$BRED
		reset=$RESET
	fi

	local fmt=$(gettext "${1:-0}")
	[[ $# -gt 0 ]] && shift

	if [[ ! $fmt =~ ^[0-9] ]]; then
		# Build the message with a colored asterisk and the format string
		local txt=$(printf "${bred} * ${reset}$fmt" "$@")
		# Print the message, right-aligning the marker [!!] at the edge of the terminal
		if [[ -t 1 ]]; then
			printf "$txt%*s ${bblue}[${bred} !! ${bblue}]${reset}\n" \
				$(( $(get_cols) - ${#fmt} - 10 )) >&2
		else
			printf "$txt [ !! ]\n" >&2
		fi
	elif [[ $fmt -gt 0 ]]; then
		# Move the cursor up one line and right by the required number
		# of columns, then print the marker
		if [[ -t 1 ]]; then
			printf "\033[A\033[%sC ${bblue}[${bred} !! ${bblue}]${reset}\n" \
				$(( $(get_cols) - 7 )) >&2
		else
			printf " [ !! ]\n" >&2
		fi
	else
		# Otherwise, print the marker "[ ok ]" (in green) 
		# at the end of the previous line
		if [[ -t 1 ]]; then
			printf "\033[A\033[%sC ${bblue}[${bgreen} ok ${bblue}]${reset}\n" \
				$(( $(get_cols) - 7 ))
		else
			printf " [ ok ]\n"
		fi
	fi
}

# Detects and returns current terminal width in columns
get_cols() {
	local cols=$(stty size 2>/dev/null | awk '{print $2}')
	[[ -z $cols ]] && cols=$(tput cols 2>/dev/null)
	[[ -z $cols ]] && cols=80
	echo $cols
}

